@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 The Bitcoin Core developers
+// Copyright (c) 2017-present The Bitcoin Core developers
 // Distributed under the MIT software license, see the accompanying
 // file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -532,7 +532,7 @@ void Num3072::ToBytes(unsigned char (&out)[BYTE_SIZE]) {
     }
 }
 
-Num3072 MuHash3072::ToNum3072(Span<const unsigned char> in) {
+Num3072 MuHash3072::ToNum3072(std::span<const unsigned char> in) {
     unsigned char tmp[Num3072::BYTE_SIZE];
 
     uint256 hashed_in{(HashWriter{} << in).GetSHA256()};
@@ -543,7 +543,7 @@ Num3072 MuHash3072::ToNum3072(Span<const unsigned char> in) {
     return out;
 }
 
-MuHash3072::MuHash3072(Span<const unsigned char> in) noexcept
+MuHash3072::MuHash3072(std::span<const unsigned char> in) noexcept
 {
     m_numerator = ToNum3072(in);
 }
@@ -573,12 +573,12 @@ MuHash3072& MuHash3072::operator/=(const MuHash3072& div) noexcept
     return *this;
 }
 
-MuHash3072& MuHash3072::Insert(Span<const unsigned char> in) noexcept {
+MuHash3072& MuHash3072::Insert(std::span<const unsigned char> in) noexcept {
     m_numerator.Multiply(ToNum3072(in));
     return *this;
 }
 
-MuHash3072& MuHash3072::Remove(Span<const unsigned char> in) noexcept {
+MuHash3072& MuHash3072::Remove(std::span<const unsigned char> in) noexcept {
     m_denominator.Multiply(ToNum3072(in));
     return *this;
 }
