@@ -13,10 +13,6 @@
         <source>&amp;New</source>
         <translation type="unfinished">&amp;Bayu</translation>
     </message>
-    <message>
-        <source>Copy the currently selected address to the system clipboard</source>
-        <translation type="unfinished">Kopyan me ing makalage address king system clipboard</translation>
-    </message>
     <message>
         <source>&amp;Copy</source>
         <translation type="unfinished">&amp;Kopyan</translation>
@@ -385,8 +381,23 @@
         <translation type="unfinished">Memali ya ing pamangaua king key</translation>
     </message>
 </context>
+<context>
+    <name>HelpMessageDialog</name>
+    <message>
+        <source>version</source>
+        <translation type="unfinished">bersion</translation>
+    </message>
+    <message>
+        <source>Command-line options</source>
+        <translation type="unfinished">Pipamilian command-line</translation>
+    </message>
+</context>
 <context>
     <name>Intro</name>
+    <message>
+        <source>Welcome</source>
+        <translation type="unfinished">Malaus ka</translation>
+    </message>
     <message numerus="yes">
         <source>%n GB of space available</source>
         <translation type="unfinished">
@@ -416,21 +427,6 @@
         <source>Error</source>
         <translation type="unfinished">Mali</translation>
     </message>
-    <message>
-        <source>Welcome</source>
-        <translation type="unfinished">Malaus ka</translation>
-    </message>
-    </context>
-<context>
-    <name>HelpMessageDialog</name>
-    <message>
-        <source>version</source>
-        <translation type="unfinished">bersion</translation>
-    </message>
-    <message>
-        <source>Command-line options</source>
-        <translation type="unfinished">Pipamilian command-line</translation>
-    </message>
 </context>
 <context>
     <name>ModalOverlay</name>
@@ -746,10 +742,6 @@
         <source>Signature</source>
         <translation type="unfinished">Pirma</translation>
     </message>
-    <message>
-        <source>Copy the current signature to the system clipboard</source>
-        <translation type="unfinished">Kopyan ing kasalungsungan pirma king system clipboard</translation>
-    </message>
     <message>
         <source>Sign the message to prove you own this Bitcoin address</source>
         <translation type="unfinished">Pirman ing mensayi ban patune na keka ya ining Bitcoin address</translation>
