@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# Copyright (c) 2022 The Bitcoin Core developers
+# Copyright (c) 2022-present The Bitcoin Core developers
 # Distributed under the MIT software license, see the accompanying
 # file COPYING or http://www.opensource.org/licenses/mit-license.php.
 
@@ -15,7 +15,10 @@ except ImportError:
     pass
 from test_framework.messages import COIN
 from test_framework.test_framework import BitcoinTestFramework
-from test_framework.util import assert_equal
+from test_framework.util import (
+    assert_equal,
+    bpf_cflags,
+)
 from test_framework.wallet import MiniWallet
 
 utxocache_changes_program = """
@@ -147,6 +150,7 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         self.skip_if_no_bitcoind_tracepoints()
         self.skip_if_no_python_bcc()
         self.skip_if_no_bpf_permissions()
+        self.skip_if_running_under_valgrind()
 
     def run_test(self):
         self.wallet = MiniWallet(self.nodes[0])
@@ -181,7 +185,7 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         ctx = USDT(pid=self.nodes[0].process.pid)
         ctx.enable_probe(probe="utxocache:uncache",
                          fn_name="trace_utxocache_uncache")
-        bpf = BPF(text=utxocache_changes_program, usdt_contexts=[ctx], debug=0, cflags=["-Wno-error=implicit-function-declaration"])
+        bpf = BPF(text=utxocache_changes_program, usdt_contexts=[ctx], debug=0, cflags=bpf_cflags())
 
         # The handle_* function is a ctypes callback function called from C. When
         # we assert in the handle_* function, the AssertError doesn't propagate
@@ -250,7 +254,7 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         ctx.enable_probe(probe="utxocache:add", fn_name="trace_utxocache_add")
         ctx.enable_probe(probe="utxocache:spent",
                          fn_name="trace_utxocache_spent")
-        bpf = BPF(text=utxocache_changes_program, usdt_contexts=[ctx], debug=0, cflags=["-Wno-error=implicit-function-declaration"])
+        bpf = BPF(text=utxocache_changes_program, usdt_contexts=[ctx], debug=0, cflags=bpf_cflags())
 
         # The handle_* function is a ctypes callback function called from C. When
         # we assert in the handle_* function, the AssertError doesn't propagate
@@ -265,14 +269,6 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         actual_utxocache_adds = []
         actual_utxocache_spents = []
 
-        def compare_utxo_with_event(utxo, event):
-            """Compare a utxo dict to the event produced by BPF"""
-            assert_equal(utxo["txid"], bytes(event.txid[::-1]).hex())
-            assert_equal(utxo["index"], event.index)
-            assert_equal(utxo["height"], event.height)
-            assert_equal(utxo["value"], event.value)
-            assert_equal(utxo["is_coinbase"], event.is_coinbase)
-
         def handle_utxocache_add(_, data, __):
             event = ctypes.cast(data, ctypes.POINTER(UTXOCacheChange)).contents
             self.log.info(f"handle_utxocache_add(): {event}")
@@ -324,9 +320,28 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
 
         self.log.info(
             f"check that we successfully traced {EXPECTED_HANDLE_ADD_SUCCESS} adds and {EXPECTED_HANDLE_SPENT_SUCCESS} spent")
-        for expected_utxo, actual_event in zip(expected_utxocache_adds + expected_utxocache_spents,
-                                               actual_utxocache_adds + actual_utxocache_spents):
-            compare_utxo_with_event(expected_utxo, actual_event)
+
+        # Check that all expected tracepoints are received, but not the order they were received in.
+        # Tracepoint ordering is not strictly guaranteed, so this comparison avoids intermittent failures in the test.
+        def cache_event_to_key(event):
+            return (
+                bytes(event.txid[::-1]).hex(),
+                event.index,
+                event.height,
+                event.value,
+                event.is_coinbase
+            )
+
+        expected_add_keys = {(e["txid"], e["index"], e["height"], e["value"], e["is_coinbase"])
+                             for e in expected_utxocache_adds}
+        expected_spent_keys = {(e["txid"], e["index"], e["height"], e["value"], e["is_coinbase"])
+                              for e in expected_utxocache_spents}
+
+        actual_add_keys = {cache_event_to_key(e) for e in actual_utxocache_adds}
+        actual_spent_keys = {cache_event_to_key(e) for e in actual_utxocache_spents}
+
+        assert_equal(expected_add_keys, actual_add_keys)
+        assert_equal(expected_spent_keys, actual_spent_keys)
 
         bpf.cleanup()
 
@@ -339,7 +354,7 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         ctx = USDT(pid=self.nodes[0].process.pid)
         ctx.enable_probe(probe="utxocache:flush",
                          fn_name="trace_utxocache_flush")
-        bpf = BPF(text=utxocache_flushes_program, usdt_contexts=[ctx], debug=0, cflags=["-Wno-error=implicit-function-declaration"])
+        bpf = BPF(text=utxocache_flushes_program, usdt_contexts=[ctx], debug=0, cflags=bpf_cflags())
 
         # The handle_* function is a ctypes callback function called from C. When
         # we assert in the handle_* function, the AssertError doesn't propagate
@@ -396,7 +411,7 @@ class UTXOCacheTracepointTest(BitcoinTestFramework):
         ctx = USDT(pid=self.nodes[0].process.pid)
         ctx.enable_probe(probe="utxocache:flush",
                          fn_name="trace_utxocache_flush")
-        bpf = BPF(text=utxocache_flushes_program, usdt_contexts=[ctx], debug=0, cflags=["-Wno-error=implicit-function-declaration"])
+        bpf = BPF(text=utxocache_flushes_program, usdt_contexts=[ctx], debug=0, cflags=bpf_cflags())
         bpf["utxocache_flush"].open_perf_buffer(handle_utxocache_flush)
 
         self.log.info("prune blockchain to trigger a flush for pruning")
