@@ -15,9 +15,6 @@ from test_framework.util import (
 )
 
 class TxConflicts(BitcoinTestFramework):
-    def add_options(self, parser):
-        self.add_wallet_options(parser)
-
     def set_test_params(self):
         self.num_nodes = 3
 
